
#include once "vbcompat.bi"

#include once "connect.bi"
#include once "fbsound\inc\fbsound.bi"
#include once "fbjson\fbjson.bi"
#include once "sgui\sgui.bas"
#include once "sgui\clickmapgadget.bas"
#include once "sgui\menu.bas"
#include once "stringlist.bi"

#include once "arbitrator\chess.bi"
#include once "colors.bi"
#include once "graphics.bi"
#include once "language.bi"
#include once "validator.bi"
#include once "version.bi"

#define _D1_ ExePath() & "\media\"
#define _D2_ ExePath() & "\source\fbsound\lib\win32\"
#define _F1_ ExePath() & "\engines.json"
#define _F2_ ExePath() & "\eschecs.log"
#define _F3_ ExePath() & "\media\illegal.wav"
#define _F4_ ExePath() & "\eco.json"

#macro OnNewPosition()
history.Add(game.FEN)
game.GenerateLegalMoves()
state = game.GameState
if cbool(state = gsInProgress) andalso DetectRepetition() then
  state = gsDraw
end if
if len(move) = 4 then chessboard.MovePiece(move)
if rookMove <> "" then chessboard.MovePiece(rookMove)
if state = gsInProgress then
  check = game.Check
  DisplayArbitratorMessage(IIF(check, sCheck, "") & IIF(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove) & IIF(ComputerToMove(), "..", ""))
else
  DisplayArbitratorMessage(sState(state))
end if
if searchOpeningName andalso (len(sequence) > 0) then
  if ecoDatabase.ContainsKey(sequence) then
    LogLn(mid(ecoDatabase[sequence].Value, 5))
  else
    searchOpeningName = false
  end if
end if
#endmacro

#macro OnLegalMove()
if IsPromotionMove(game, move) then
  if comesFrom = username then
    if     GetChecked(event, 4, 1) then
      promotion = chess.cKnight
    elseif GetChecked(event, 4, 2) then
      promotion = chess.cBishop
    elseif GetChecked(event, 4, 3) then
      promotion = chess.cRook
    elseif GetChecked(event, 4, 4) then
      promotion = chess.cQueen
    end if
  else
    var i = instr("nbrq", mid(move, 5, 1))
    if (len(move) = 5) andalso (i > 0) then
      select case i
        case 1 : promotion = chess.cKnight
        case 2 : promotion = chess.cBishop
        case 3 : promotion = chess.cRook
        case 4 : promotion = chess.cQueen
      end select
    else
      promotion = chess.cQueen
    end if
  end if
  game.DoMove(move, promotion)
  chessboard.Promote(left(move, 2), promotionSymbol[promotion])
else
  if IsCastlingMove(game, move) then
    select case move
      case "e1g1" : rookMove = "h1f1"
      case "e1c1" : rookMove = "a1d1"
      case "e8g8" : rookMove = "h8f8"
      case "e8c8" : rookMove = "a8d8"
    end select
  elseif IsEnPassantMove(game, move) then
    chessboard.HidePiece(chr(move[2], move[1]))
  end if
  game.DoMove(move)
end if
LogLn(move & " " & comesFrom)
if searchOpeningName then
  if len(sequence) < 100 then
    sequence &= move
  else
    sequence = ""
    searchOpeningName = false
  end if
end if
#endmacro

#macro OnTryConnect()
if engineConnected then
  Sleep(500)
  SendCommand("uci")
else
  listener->quit = true
  autoPlay = false
  Menu(event, 2, 2, 2, sAutoplay)
  LogLn("Connection failed.")
end if
#endmacro

using sGUI
using strlist

declare sub DisplayArbitratorMessage(byval as string)
declare function IsPromotionMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsCastlingMove(byref as chess.tChessGame, byval as string) as boolean
declare function IsEnPassantMove(byref as chess.tChessGame, byval as string) as boolean
declare function ReplaceFieldName(byval as string) as string
declare function ComparePiecePlacement(byval as string, byval as string) as boolean
declare function ComputerToMove() as boolean
declare function LoadFileAsString(byval filename as string) as string
declare sub LoadEnginesData(aFileName as const string)
declare function ConnectEngine(aIndex as const integer) as boolean
declare sub ProcedureThread(byval param as any ptr)
declare sub OnOutput(aOutput as const string)
declare sub OnUciOk(aOutput as const string)
declare sub OnBestMove(aOutput as const string)
declare function DetectRepetition() as boolean
declare sub SetWindowTitle(aLoadedEngineName as const string)

type TListener
  dim handle as any ptr = 0
  dim sync as any ptr = 0
  dim quit as boolean = false
  dim procedure as sub (aOutput as const string) = 0
end type

type chessfield 
  fieldname as string 
  selected as integer 
  posx as integer 
  posy as integer 
end type

type TEngineInfo
  as string eiCommand, eiName, eiWorkingDirectory
  as boolean eiExists
end type

const textHeight = 14
dim as integer w, h
ScreenRes 320, 18 + 320 + 18, 32,, 4
ScreenInfo w, h
SetWindowTitle("")
Width w \ 8, h \ textHeight
SetLanguage()

dim shared as uinteger messageTextColor
dim shared as uinteger messageColor
customColors = true
backgroundColor	= colors.white
textColor	= colors.black
gadgetColor	= colors.teagreen
menuColor = colors.darkteagreen
gadgetGlowColor	= colors.magenta
gadgetTextColor	= colors.black
cursorColor = colors.magenta
menuTextColor = colors.black
menuHiliteColor = gadgetColor
menuGhostedColor = ShiftRGB(MenuColor, -contrast / 2)
messageTextColor = textColor
messageColor = gadgetColor
graph.resourcePath = _D1_
graph.boardLeft = 0
graph.boardTop = 18
InitGFX
DisplayArbitratorMessage("")

dim shared as EventHandle ptr event
event = new EventHandle

dim as gadget ptr boardClickSensor  = AddClickMapGadget( _
  event, _
  graph.boardLeft, _
  graph.boardTop, _
  graph.resourcePath & "clickmap.bmp" _
)
GadgetOn(boardClickSensor)

dim as integer field_ID
dim as chessfield Board(1 to 64)

for k as integer = 0 to 7 
  Board(1 + k * 8).fieldname = "a" & str(k + 1) 
  Board(2 + k * 8).fieldname = "b" & str(k + 1) 
  Board(3 + k * 8).fieldname = "c" & str(k + 1) 
  Board(4 + k * 8).fieldname = "d" & str(k + 1) 
  Board(5 + k * 8).fieldname = "e" & str(k + 1) 
  Board(6 + k * 8).fieldname = "f" & str(k + 1) 
  Board(7 + k * 8).fieldname = "g" & str(k + 1) 
  Board(8 + k * 8).fieldname = "h" & str(k + 1) 
next k

dim shared as boolean autoPlay = true
dim shared as integer autoColor = chess.cBlack
dim shared engines() as TEngineInfo
LoadEnginesData(_F1_)

dim engineIndex as integer = 0 ' <---

dim as integer _
  item_quit, _
  item_move, _
  item_auto, _
  item_new, _
  item_flip, _
  item_knight, _
  item_bishop, _
  item_rook, _
  item_queen, _
  item()

              Menu(event, 1, 0, 1, PROGRAM_NAME)
item_quit   = Menu(event, 1, 1, 1, sQuit)
              Menu(event, 2, 0, 1, sMoves)
item_move   = Menu(event, 2, 1, 1, sMove)
item_auto   = Menu(event, 2, 2, IIF(autoplay, 3, 1), sAutoplay)
if ubound(engines) - lbound(engines) >= 0 then
              Menu(event, 2, 3, 4, "")
  redim item(lbound(engines) to ubound(engines))
  for i as integer = lbound(engines) to ubound(engines)
    if (i = engineIndex) andalso (not engines(i).eiExists) andalso (i < ubound(engines)) then
      engineIndex += 1
    end if
    item(i) = Menu(event, 2, i + 4, IIF(i = engineIndex, 3, 1) + IIF(engines(i).eiExists, 0, -1), engines(i).eiName)
  next i
end if
              Menu(event, 3, 0, 1, sBoard)
item_new    = Menu(event, 3, 1, 1, sNew)
item_flip   = Menu(event, 3, 2, 1, sFlip)
              Menu(event, 4, 0, 1, sPromotion)
item_knight = Menu(event, 4, 1, 1, sKnight)
item_bishop = Menu(event, 4, 2, 1, sBishop)
item_rook   = Menu(event, 4, 3, 1, sRook)
item_queen  = Menu(event, 4, 4, 3, sQueen)
MenuOn(event)

graph.upsideDown = false
dim startingPosition as string = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"
if (len(Command(1)) > 0) andalso IsFEN(Command(1)) then startingPosition = Command(1)
dim shared chessboard as graph.TGraphicalChessboard
chessboard.Create()
chessboard.ChangePieces(mid(startingPosition, 1, instr(startingPosition, " ") - 1))
chessboard.RedrawPieces()
dim shared game as chess.tChessGame
dim shared move as string
dim shared state as tGameState = gsCheckmate
dim shared check as boolean = false
dim shared history as stringlist
dim shared rookMove as string
dim shared promotion as integer
dim shared oneMove as boolean = false
dim shared comesFrom as string
dim shared username as string
dim shared promotionSymbol as string
username = Environ("USERNAME")
promotionSymbol = "  nbrq"
dim shared as string vEngineName, vAuthorName
dim shared regExprCompiled as boolean
regExprCompiled = uci.CompileRegExpr()
dim shared appLog as integer
appLog = FreeFile()
open _F2_ for output as #appLog
#define LogLn(x) print #appLog, Time() & " " & x
LogLn(PROGRAM_NAME & " " & PROGRAM_VERSION & " build " & __DATE_ISO__ & " " & __FB_SIGNATURE__)

dim shared as string sequence
sequence = ""
dim shared as jsonItem ecoDatabase
ecoDatabase = jsonItem(Trim(LoadFileAsString(_F4_), any Chr(13, 10)))
dim shared as boolean searchOpeningName
searchOpeningName = (ecoDatabase.DataType = jsonObject)

game.Initialize(startingPosition)
OnNewPosition()

dim listener as TListener ptr = new TListener
listener->sync = mutexcreate
listener->procedure = @OnOutput
listener->handle = threadcreate(@ProcedureThread, listener)

dim engineConnected as boolean = ConnectEngine(engineIndex)
OnTryConnect()

dim shared waiting as boolean
waiting = false

dim as integer hWaveIllegal
fbs_Set_PlugPath(_D2_)
fbs_Init()
fbs_Load_WAVFile(_F3_, @hWaveIllegal)

const MT = "1000"

do
  event->xSleep(1)
  
  if engineConnected _
  andalso ComputerToMove() then
    if not waiting then
      oneMove = false
      SendCommand("position fen " & game.FEN())
      SendCommand("go movetime " & MT)
      mutexlock(listener->sync)
      waiting = true
      mutexunlock(listener->sync)
    end if
  end if
  
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case boardClickSensor
        if waiting then
          continue do
        end if
        if state = gsInProgress then
          field_ID = getClickMapColor(boardClickSensor)
          
          move += IIF(graph.upsideDown, ReplaceFieldName(Board(field_ID).fieldname), Board(field_ID).fieldname)
          
          if len(move) = 2 then
            if game.IsLegalStartingSquare(move) then
              chessboard.Backup()
              var dest = game.LegalDestinations(move)
              screenlock
              for i as integer = 0 to Len(dest) \ 2 - 1
                chessboard.Highlight(mid(dest, 2 * i + 1, 2))
              next
              screenunlock
            else
              fbs_Play_Wave(hWaveIllegal)
              move = ""
            end if  
          end if
          if len(move) = 4 then
            chessboard.Restore_()
            if game.IsLegal(move) then
              comesFrom = username
              OnLegalMove()
              OnNewPosition()
            else
              fbs_Play_Wave(hWaveIllegal)
            end if
            move = ""
            rookMove = ""
          end if
        end if
    end select
  end if
  
  if event->MENUMESSAGE then
    select case event->MENUMESSAGE
      case item_quit
        event->EXITEVENT = 1
      case item_move
        if autoPlay then
          autoColor = chess.cBlack * autoColor
        else
          oneMove = true
        end if
      case item_auto
        if GetChecked(event, 2, 2) then
          Menu(event, 2, 2, 1, sAutoplay)
        else
          Menu(event, 2, 2, 3, sAutoplay)
        end if
        autoPlay = GetChecked(event, 2, 2)
      case item_new
        chessboard.ClearBoard()
        chessboard.ChangePieces()
        chessboard.RedrawPieces()
        game.Initialize()
        history.ClearList
        sequence = ""
        searchOpeningName = (ecoDatabase.DataType = jsonObject)
        OnNewPosition()
      case item_flip
        chessboard.ClearBoard()
        graph.upsideDown = not graph.upsideDown
        for i as integer = 0 to chessboard.number - 1
          chessboard.array(i).img.fLeft = graph.XToScreen(chessboard.array(i).fX)
          chessboard.array(i).img.fTop = graph.YToScreen(chessboard.array(i).fY)
        next i
        chessboard.RedrawPieces()
      case item_knight
        Menu(event, 4, 1, 3, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_bishop
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 3, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_rook
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 3, sRook)
        Menu(event, 4, 4, 1, sQueen)
      case item_queen
        Menu(event, 4, 1, 1, sKnight)
        Menu(event, 4, 2, 1, sBishop)
        Menu(event, 4, 3, 1, sRook)
        Menu(event, 4, 4, 3, sQueen)
      case else
        for i as integer = lbound(item) to ubound(item)
          if event->MENUMESSAGE = item(i) then
            engineIndex = i
            for j as integer = lbound(item) to ubound(item)
              Menu(event, 2, j + 4, IIF(j = engineIndex, 3, 1) + IIF(engines(j).eiExists, 0, -1), engines(j).eiName)
            next j
            SendCommand("quit")
            listener->quit = true
            threadwait(listener->handle)
            delete listener
            listener = new TListener
            listener->procedure = @OnOutput
            listener->handle = threadcreate(@ProcedureThread, listener)
            engineConnected = ConnectEngine(engineIndex)
            OnTryConnect()
          end if
        next i
    end select
  end if
loop until event->EXITEVENT

if engineConnected then
  SendCommand("quit")
end if

delete event
chessboard.Free
close #appLog
listener->quit = true
threadwait(listener->handle)
mutexdestroy(listener->sync)
delete listener

sub DisplayArbitratorMessage(byval s as string = "")
  line (0, 18 + 320)-(320, 18 + 320 + 18), messageColor, BF
  draw string (8, 18 + 320 + (18 - textHeight) \ 2), s, messageTextColor
end sub

function IsPromotionMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

function IsCastlingMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cKing) andalso (Abs(m[0] - m[2]) = 2)
end function

function IsEnPassantMove(byref g as chess.tChessGame, byval m as string) as boolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = chess.cPawn) andalso ((abs(g.board(m[2] - 96, m[3] - 48)) = chess.cNothing) and (m[0] <> m[2]))
end function

function ReplaceFieldName(byval s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  return chr(asc1 + 96, asc2 + 48)
end function

function ComparePiecePlacement(byval FENRecord1 as string, byval FENRecord2 as string) as boolean
  return mid(FENRecord1, 1, instr(FENRecord1, " ")) = mid(FENRecord2, 1, instr(FENRecord2, " "))
end function

function ComputerToMove() as boolean
  return _
    cbool(state = gsInProgress) _
    andalso ((autoPlay andalso cbool(autoColor = game.activeColor)) or oneMove)
end function

function LoadFileAsString(byval filename as string) as string
  dim s as string
  dim vFile as integer = FreeFile()
  if open(filename for input access read as #vFile) = 0 then
    close #vFile
    if open(filename for binary access read as #vFile) = 0 then
      s = space(lof(vFile))
      get #vFile,, s
      close #vFile
    end if 
  end if
  return s
end function

sub LoadEnginesData(aFileName as const string)
  dim as string jsontext = Trim(LoadFileAsString(aFileName), any Chr(13, 10))
  dim as jsonItem item1, item = jsonItem(jsontext)
  dim ei as TEngineInfo
  if item.Datatype = jsonArray then
    redim engines(item.Count - 1)
    for i as integer = 0 to item.Count - 1
      item1 = jsonItem(item[i])
      if item[i].DataType = jsonObject then
        with ei
          .eiCommand = item1["command"].Value
          .eiName = item1["name"].Value
          .eiWorkingDirectory = item1["workingDirectory"].Value
          .eiExists = FileExists(ExePath() & ei.eiWorkingDirectory & ei.eiCommand)
        end with
        engines(i) = ei
      end if
    next i
  end if
end sub

sub ProcedureThread(byval param as any ptr)
  dim engineOutput as string
  with *cast(TListener ptr, param)
    do
      Sleep(10)
      engineOutput = TryGetAnswer()
      if len(engineOutput) > 0 then
        MutexLock(.sync)
        .procedure(engineOutput)
        MutexUnlock(.sync)
      end if
    loop until .quit
  end with
end sub

sub OnOutput(aOutput as const string)
  if instrrev(aOutput, "uciok") > 0 then
    OnUciOk(aOutput)
  elseif instrrev(aOutput, "bestmove") > 0 then
    OnBestMove(aOutput)
  end if
end sub

sub OnUciOk(aOutput as const string)
  if uci.ParseUciOk(aOutput, vEngineName, vAuthorName) then
    LogLn("Engine connected")
    LogLn("Engine name """ & vEngineName & """")
    LogLn("Engine author """ & vAuthorName & """")
    SetWindowTitle(vEngineName)
  end if
end sub

sub OnBestMove(aOutput as const string)
  move = ExtractBestMove(aOutput)
  LogLn("Engine move """ & move & """")
  if cbool(len(move) >= 4) andalso game.IsLegal(move) then
    LogLn("Engine move is legal")
    comesFrom = vEngineName
    OnLegalMove()
    OnNewPosition()
  else
    LogLn("Engine move is not legal")
    autoPlay = false
    if GetChecked(event, 2, 2) then Menu(event, 2, 2, 1, sAutoplay)
    DisplayArbitratorMessage(sEngineError & IIF(game.activeColor = chess.cWhite, sWhiteToMove, sBlackToMove))
  end if
  move = ""
  rookMove = ""
  waiting = false
end sub

function ConnectEngine(aIndex as const integer) as boolean
  return cbool(aIndex >= lbound(engines)) _
  andalso cbool(aIndex <= ubound(engines)) _
  andalso cbool(ChDir(ExePath() & engines(aIndex).eiWorkingDirectory) = 0) _
  andalso Connect(engines(aIndex).eiCommand) _
  andalso regExprCompiled
end function

function DetectRepetition() as boolean
  dim as integer vCount = 0
  for i as integer = 0 to history.Count - 2
    if ComparePiecePlacement(history.Strings(i), history.Strings(history.Count - 1)) then vCount += 1
  next
  return cbool(vCount = 2)
end function

sub SetWindowTitle(aLoadedEngineName as const string)
  WindowTitle(PROGRAM_NAME & " " & iif(len(aLoadedEngineName) > 0, "[" & aLoadedEngineName & "]", PROGRAM_VERSION))
end sub
