#ifndef __FBSOUND_BI__
#define __FBSOUND_BI__

'  ##############
' # fbsound.bi #
'##############
' Copyright 2005 - 2018 by D.J.Peters (Joshy)
' d.j.peters@web.de

#include once "fbstypes.bi"
#include once "fbscpu.bi"

#ifndef NODSP
#include once "fbsdsp.bi"
#endif

#ifndef NOMOD
#include once "dumb.bi"
#endif

#ifndef NOMP3
#include once "mad.bi"
#endif

#ifndef NOOGG
#include once "vorbis.bi"
#endif


#inclib "fbsound"

declare function FBS_Init(byval nRate        as integer=44100, _
                          byval nChannels    as integer=    2, _
                          byval nBuffers     as integer=    3, _
                          byval nFrames      as integer= 2048, _
                          byval nPlugIndex   as integer=    0, _
                          byval nDeviceIndex as integer=    0) as boolean

' now fbs will start,stop and exit by it self
declare function FBS_Start() as boolean
declare function FBS_Stop() as boolean
declare function FBS_Exit() as boolean

declare function FBS_Get_PlugPath() as string
declare sub      FBS_Set_PlugPath(byval NewPath as string)

declare function FBS_Get_NumOfPlugouts() as integer
declare function FBS_Get_PlugName() as string
declare function FBS_Get_PlugDevice() as string
declare function FBS_Get_PlugError() as string
declare function FBS_Get_PlugRate() as integer       ' 6000Hz-96000Hz
declare function FBS_Get_PlugBits() as integer       ' signed 16 bit
declare function FBS_Get_PlugChannels() as integer   ' 1=mono 2=stereo
declare function FBS_Get_PlugBuffers() as integer    ' 2 to N N<=64
declare function FBS_Get_PlugBuffersize() as integer ' same as FrameSize*Frames
declare function FBS_Get_PlugFrames() as integer     ' same as BufferSize\FrameSize
declare function FBS_Get_PlugFramesize() as integer  ' same as BufferSize\Frames
declare function FBS_Get_PlugRunning() as boolean

declare function FBS_Get_PlayingSounds() as integer

declare function FBS_Get_PlayedBytes() as integer
declare function FBS_Get_PlayedSamples() as integer
declare function FBS_Get_PlayTime() as double


declare function FBS_Get_MasterVolume(byval lpVolume as single ptr) as boolean
declare function FBS_Set_MasterVolume(byval Volume   as single ) as boolean

#ifndef NODSP
declare function FBS_Set_MasterFilter(byval nFilter as integer, _
                                      byval Center  as single, _
                                      byval dB      as single, _
                                      byval Octave  as single = 1.0, _
                                      byval OnOff   as boolean = True) as boolean

declare function FBS_Enable_MasterFilter(byval nFilter as integer) as boolean
declare function FBS_Disable_MasterFilter(byval nFilter as integer) as boolean

declare sub      FBS_PitchShift(byval d as short ptr, _
                                byval s as short ptr, _
                                byval v as single   , _
                                byval n as integer  )
#endif

declare function FBS_Get_MaxChannels(byval pnChannels as integer ptr) as boolean
declare function FBS_Set_MaxChannels(byval nChannels  as integer ) as boolean

#ifndef NOCALLBACK
declare function FBS_Set_LoadCallback(byval cb as FBS_LOADCALLBACK) as boolean
declare function FBS_Enable_LoadCallback() as boolean
declare function FBS_Disable_LoadCallback() as boolean

declare function FBS_Set_MasterCallback(byval pCallback as FBS_BUFFERCALLBACK) as boolean
declare function FBS_Enable_MasterCallback() as boolean
declare function FBS_Disable_MasterCallback() as boolean
#endif

' create or load wave objects in the pool of Waves()

' create hWave from *.wav file
declare function FBS_Load_WAVFile(byref Filename as string , _
                                  byval phWave   as integer ptr) as boolean
#ifndef NOMP3
' create hWave from *.mp3,*.mp2,*.mp file
declare function FBS_Load_MP3File(byref Filename as string      , _
                                  byval phWave  as integer ptr , _
                                  byref _usertmpfile_  as string ="") as boolean
#endif

#ifndef NOMOD
' create hWave from *.it *.xm *.sm3 or *.mod file
declare function FBS_Load_MODFile(byref Filename as string       , _
                                  byval phWave  as integer ptr) as boolean
#endif

#ifndef NOOGG
' create hWave from *.ogg file
declare function FBS_Load_OGGFile(byref Filename as string      , _
                                  byval lphWave  as integer ptr , _
                                  byref _usertmpfile_  as string ="") as boolean
#endif
' create hWave with nSamples in memory
declare function FBS_Create_Wave(byval nSamples as integer     , _
                                 byval phWave   as integer ptr , _
                                 byval ppWave   as any ptr ptr) as boolean
' playtime in MS
declare function FBS_Get_WaveLength(byval hWave as integer, _
                                    byval pMS   as integer ptr) as boolean



' play any wave as sound from the pool of Waves()
' optional number of loops, playbackspeed,volume,pan
' if you need to change any param while playing use an hSound object
declare function FBS_Play_Wave(byval hWave    as integer        , _
                               byval nLoops   as integer  = 1   , _
                               byval Speed    as single  = 1.0 , _
                               byval Volume   as single  = 1.0 , _
                               byval Pan      as single  = 0.0 , _
                               byval phSound  as integer ptr = NULL) as boolean

'create an playable sound object "hSound" from any "hWave" object
declare function FBS_Create_Sound(byval hWave   as integer        , _
                                  byval phSound as integer ptr = NULL) as boolean


' [optinal] destroy/free created hSound's and hWave's
declare function FBS_Destroy_Wave(byval phWave  as integer ptr) as boolean
declare function FBS_Destroy_Sound(byval phSound as integer ptr) as boolean

' play an hSound object
declare function FBS_Play_Sound(byval hSound as integer , _
                                byval nLoops as integer = 1) as boolean
' play time in MS
declare function FBS_Get_SoundLength(byval hSound as integer , _
                                     byval lpMS   as integer ptr) as boolean


' get and set any params from playing hSound
declare function FBS_Set_SoundSpeed(byval hSound as integer , _
                                    byval Speed  as single) as boolean
declare function FBS_Get_SoundSpeed(byval hSound as integer , _
                                    byval pSpeed as single ptr) as boolean

declare function FBS_Set_SoundVolume(byval hSound as integer , _
                                     byval Volume as single) as boolean
declare function FBS_Get_SoundVolume(byval hSound as integer, _
                                     byval pVolume as single ptr) as boolean

declare function FBS_Set_SoundPan(byval hSound as integer, _
                                  byval Pan    as single) as boolean
declare function FBS_Get_SoundPan(byval hSound as integer        , _
                                  byval pPan   as single ptr     ) as boolean

declare function FBS_Set_SoundLoops(byval hSound as integer , _
                                    byval nLoops as integer) as boolean
declare function FBS_Get_SoundLoops(byval hSound  as integer , _
                                    byval pnLoops as integer ptr) as boolean
' togle hearing
declare function FBS_Set_SoundMuted(byval hSound as integer , _
                                    byval muted  as boolean) as boolean
declare function FBS_Get_SoundMuted(byval hSound as integer , _
                                    byval pMuted as boolean ptr ) as boolean
' togle playing
declare function FBS_Set_SoundPaused(byval hSound as integer , _
                                     byval Paused as boolean) as boolean
declare function FBS_Get_SoundPaused(byval hSound as integer        , _
                                     byval pPaused as boolean ptr ) as boolean

declare function FBS_Get_WavePointers(byval hWave       as integer            , _
                                      byval ppWaveStart as short ptr ptr=NULL , _
                                      byval ppWaveEnd   as short ptr ptr=NULL , _
                                      byval pnChannels  as integer ptr  =NULL ) as boolean

declare function FBS_Get_SoundPointers(byval hSound  as integer       , _
                                       byval ppStart as short ptr ptr=NULL , _
                                       byval ppPlay  as short ptr ptr=NULL , _
                                       byval ppEnd   as short ptr ptr=NULL) as boolean

declare function FBS_Set_SoundPointers(byval hSound    as integer , _
                                       byval pNewStart as short ptr=NULL, _
                                       byval pNewPlay  as short ptr=NULL, _
                                       byval pNewEnd   as short ptr=NULL) as boolean

' position 0.0-1.0
declare function FBS_Get_SoundPosition(byval hSound as integer, _
                                       byval pPosition as single ptr) as boolean

#ifndef NOCALLBACK
declare function FBS_Set_SoundCallback(byval hSound as integer      , _
                                       byval cb     as FBS_BUFFERCALLBACK)  as boolean
declare function FBS_Enable_SoundCallback(byval hSound as integer) as boolean
declare function FBS_Disable_SoundCallback(byval hSound as integer) as boolean
#endif

#if defined(NOMP3) and defined(NOMOD)
 ' no streams
#else

enum FBS_STREAM_TYPES
  FBS_NOSTREAM =0
 #ifndef NOMOD
  FBS_MOD = 1
 #endif
 #ifndef NOMP3
  FBS_MP3 = 2
 #endif
end enum

declare function FBS_Get_PlayingStreams() as integer

 #ifndef NOMP3
declare function FBS_Create_MP3Stream(byref Filename as string) as boolean
declare function FBS_Play_MP3Stream(byval Volume as single=1.0, _
                                    byval Pan    as single=0.0) as boolean
declare function FBS_End_MP3Stream() as boolean
declare function FBS_Set_MP3StreamVolume(byval Volume as single) as boolean
declare function FBS_Get_MP3StreamVolume(byval pVolume as single ptr   ) as boolean
declare function FBS_Set_MP3StreamPan(byval Pan as single) as boolean
declare function FBS_Get_MP3StreamPan(byval pPan as single ptr) as boolean

declare function FBS_Get_MP3StreamBuffer(byval ppBuffer   as short ptr ptr , _
                                         byval pChannels as integer ptr   , _
                                         byval pnSamples as integer ptr) as boolean
  #ifndef NOCALLBACK
declare function FBS_Set_MP3StreamCallback(byval cb as FBS_BUFFERCALLBACK)  as boolean
declare function FBS_Enable_MP3StreamCallback() as boolean
declare function FBS_Disable_MP3StreamCallback() as boolean
  #endif
 #endif ' NOMP3

 #ifndef NOMOD
declare function FBS_Create_MODStream(byref Filename as string) as boolean
declare function FBS_Play_MODStream(byval Volume as single=1.0, _
                                    byval Pan    as single=0.0) as boolean
declare function FBS_End_MODStream() as boolean
declare function FBS_Set_MODStreamVolume(byval Volume     as single) as boolean
declare function FBS_Get_MODStreamVolume(byval pVolume    as single ptr   ) as boolean
declare function FBS_Set_MODStreamPan(byval Pan as single) as boolean
declare function FBS_Get_MODStreamPan(byval pPan as single ptr) as boolean

declare function FBS_Get_MODStreamBuffer(byval ppBuffer  as short ptr ptr , _
                                         byval pChannels as integer ptr   , _
                                         byval pnSamples as integer ptr) as boolean
  #ifndef NOCALLBACK
declare function FBS_Set_MODStreamCallback(byval cb as FBS_BUFFERCALLBACK)  as boolean
declare function FBS_Enable_MODStreamCallback() as boolean
declare function FBS_Disable_MODStreamCallback() as boolean
  #endif
 #endif ' NOMP3

#endif ' NOMP3 or NOMOD

#endif ' __FBSOUND_BI__
